'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.hasWdioSyncSupport = exports.wrapCommand = exports.runFnInFiberContext = exports.runTestInFiberContext = exports.executeHooksWithArgs = undefined;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _sourceMapSupport2 = require('source-map-support');

var _wdioLogger = require('wdio-logger');

var _wdioLogger2 = _interopRequireDefault(_wdioLogger);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

(0, _sourceMapSupport2.install)();


var log = (0, _wdioLogger2.default)('wdio-config');
var NOOP = function NOOP() {};

var executeHooksWithArgs = exports.executeHooksWithArgs = function () {
    var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(hooks, args) {
        return _regenerator2.default.wrap(function _callee$(_context) {
            while (1) {
                switch (_context.prev = _context.next) {
                    case 0:
                        /**
                         * make sure hooks are an array of functions
                         */
                        if (!Array.isArray(hooks)) {
                            hooks = [hooks];
                        }

                        /**
                         * make sure args is an array since we are calling apply
                         */
                        if (!Array.isArray(args)) {
                            args = [args];
                        }

                        hooks = hooks.map(function (hook) {
                            return new _promise2.default(function (resolve) {
                                var result = void 0;

                                try {
                                    result = hook.apply(null, args);
                                } catch (e) {
                                    log.error(e.stack);
                                    return resolve();
                                }

                                /**
                                 * if a promise is returned make sure we don't have a catch handler
                                 * so in case of a rejection it won't cause the hook to fail
                                 */
                                if (result && typeof result.then === 'function') {
                                    return result.then(resolve, function (e) {
                                        log.error(e.stack);
                                        resolve();
                                    });
                                }

                                resolve(result);
                            });
                        });

                        return _context.abrupt('return', _promise2.default.all(hooks));

                    case 4:
                    case 'end':
                        return _context.stop();
                }
            }
        }, _callee, this);
    }));

    function executeHooksWithArgsShim(_x, _x2) {
        return _ref.apply(this, arguments);
    }

    return executeHooksWithArgsShim;
}();

var runTestInFiberContext = exports.runTestInFiberContext = NOOP;
var runFnInFiberContext = exports.runFnInFiberContext = function runFnInFiberContext(fn, done) {
    return function () {
        for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
            args[_key] = arguments[_key];
        }

        var result = fn.apply(this, args);

        if (typeof done === 'function') {
            return done(result);
        }

        return result;
    };
};
var wrapCommand = exports.wrapCommand = function wrapCommand(_, origFn) {
    return origFn;
};
var hasWdioSyncSupport = exports.hasWdioSyncSupport = false;

/**
 * shim to make sure that we only wrap commands if wdio-sync is installed as dependency
 */
try {
    // eslint-disable-next-line import/no-unresolved
    var wdioSync = require('wdio-sync');
    log.debug('wdio-sync found, running tests synchronous');

    exports.hasWdioSyncSupport = hasWdioSyncSupport = true;
    exports.runFnInFiberContext = runFnInFiberContext = wdioSync.runFnInFiberContext;
    exports.runTestInFiberContext = runTestInFiberContext = wdioSync.runTestInFiberContext;
    exports.wrapCommand = wrapCommand = wdioSync.wrapCommand;
    exports.executeHooksWithArgs = executeHooksWithArgs = wdioSync.executeHooksWithArgs;
} catch (e) {
    log.debug('wdio-sync not found, running tests asynchronous');
}